/*
 * Project: VizKit
 * Version: 3.3.5
 
 * Date: 20141018
 * File: VisualCustomization.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2014 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "VisualCustomization.h"
#include "VisualNotification.h"
#include "VisualStageControl.h"
#include "VisualActor.h"
#include "UpdateServiceActor.h"
#include "VisualUpdateManager.h"
#include "VisualActor.h"
#include "BeatlightActor.h"
#include "CoverArtActor.h"
#include "ProcessMonitorActor.h"
#include "TemplateActor.h"
#include "TrackTitleActor.h"
#include "TrackLyricsActor.h"
#include "UpdateServiceActor.h"
#include "VisualPreferences.h"


using namespace VizKit;


void VisualCustomization::initEnsembleMembers()
{
    bool doLoadCoverArtActor = true;
	bool doLoadProcessMonitorActor = true;
	bool doLoadTrackTitleActor = true;
	bool doLoadBeatlightActor = true;
	bool doLoadTrackLyricsActor = true;
	bool doLoadTemplateActor = true;
	bool doLoadUpdateServiceActor = true;
    
    VisualActor *aVisualActor = NULL;
    
	// CoverArtActor
	if (doLoadCoverArtActor) {
		aVisualActor = new CoverArtActor;
		aVisualActor->init();
        VisualStageControl::addEnsembleMember(aVisualActor);
	}
    
	// ProcessMonitorActor
	if (doLoadProcessMonitorActor) {
		aVisualActor = new ProcessMonitorActor;
		aVisualActor->init();
		VisualStageControl::addEnsembleMember(aVisualActor);
	}
    
	// TrackTitleActor
	if (doLoadTrackTitleActor) {
		aVisualActor = new TrackTitleActor;
		aVisualActor->init();
		VisualStageControl::addEnsembleMember(aVisualActor);
	}
    
	// BeatlightActor
	if (doLoadBeatlightActor) {
		aVisualActor = new BeatlightActor;
		aVisualActor->init();
		VisualStageControl::addEnsembleMember(aVisualActor);
	}
    
	// TrackLyricsActor
	if (doLoadTrackLyricsActor) {
		aVisualActor = new TrackLyricsActor;
		aVisualActor->init();
		VisualStageControl::addEnsembleMember(aVisualActor);
	}
	
	// TemplateActor
	if (doLoadTemplateActor) {
		aVisualActor = new TemplateActor;
		aVisualActor->init();
		VisualStageControl::addEnsembleMember(aVisualActor);
	}
	
	// UpdateServiceActor
	if (doLoadUpdateServiceActor) {
		aVisualActor = new UpdateServiceActor;
		aVisualActor->init();
		VisualStageControl::addEnsembleMember(aVisualActor);
	}
}


void VisualCustomization::setDefaultPreferenceValues()
{
    // custom preference values
}


bool VisualCustomization::handleKeyPressed(const char aKeyboadCharacterValue) 
{
    bool didHandleTheKeyPress = false;
	
	switch (aKeyboadCharacterValue) {
		case 's':
		case 'S':
        {
                VisualNotification aNotification;
				aNotification.setKey(kToggleShowProcessMonitorMsg);
				VisualStageControl::dispatchNotification(aNotification);
                didHandleTheKeyPress = true;
        }
			break;						
		case 'a':
		case 'A':
            {
                VisualNotification aNotification;
				aNotification.setKey(kToggleProcessMonitorAudioInfoMsg);
				VisualStageControl::dispatchNotification(aNotification);
                didHandleTheKeyPress = true;
			}
			break;
		case 'i':
		case 'I':
        {
            VisualActor* aVisualActor = VisualStageControl::getVisualActorByName("UPDATESERVICE");
            if (aVisualActor) 
            {
                UpdateServiceActor* anUpdateServiceActor = dynamic_cast<UpdateServiceActor*>(aVisualActor);
                if (anUpdateServiceActor) 
                {
                    VisualActorState actorState = anUpdateServiceActor->getState();
                    if (actorState == kVisActOn) 
                    {
                        VisualUpdateManager::getMoreInfoAboutAvailableUpdate();
                        didHandleTheKeyPress = true;
                    }
                }
            }
        }
			break;
		case 'd':
		case 'D':
        {
            VisualActor* aVisualActor = VisualStageControl::getVisualActorByName("UPDATESERVICE");
            if (aVisualActor) 
            {
                UpdateServiceActor* anUpdateServiceActor = dynamic_cast<UpdateServiceActor*>(aVisualActor);
                if (anUpdateServiceActor) 
                {
                    VisualActorState actorState = anUpdateServiceActor->getState();
                    if (actorState == kVisActOn) 
                    {
                        VisualUpdateManager::doDownloadAvailableUpdate();
                        didHandleTheKeyPress = true;
                    }
                }
            }
        }
			break;
            
		default:
			// nothing
			break;
	}
    
    return didHandleTheKeyPress;
    
}
