/*
 * Project: VizKit
 * Version: 3.3.5
 
 * Date: 20141018
 * File: VisualMain.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2014 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "VisualMain.h"
#include "VisualMainAction.h"
#include "VisualSignature.h"
#include "VisualConfiguration.h"
#include "VisualCustomization.h"

#include <stdio.h>

using namespace VizKit;



void VisualMain::convertVisualPluginMessageToString(const OSType aVisualPluginMessage, char* anOutString) 
{
	const char* aMessageString;
	switch (aVisualPluginMessage) 
    {
		case kVisualPluginInitMessage:
			aMessageString = "kVisualPluginInitMessage";
			break;
		case kVisualPluginCleanupMessage:
			aMessageString = "kVisualPluginCleanupMessage";
			break;
		case kVisualPluginIdleMessage:
			aMessageString = "kVisualPluginIdleMessage";
			break;
		case kVisualPluginConfigureMessage:
			aMessageString = "kVisualPluginConfigureMessage";
			break;
		case kVisualPluginEnableMessage:
			aMessageString = "kVisualPluginEnableMessage";
			break;
		case kVisualPluginDisableMessage:
			aMessageString = "kVisualPluginDisableMessage";
			break;
		case kVisualPluginActivateMessage:
			aMessageString = "kVisualPluginActivateMessage";
			break;
  		case kVisualPluginWindowChangedMessage:
			aMessageString = "kVisualPluginWindowChangedMessage";
			break;
  		case kVisualPluginDeactivateMessage:
			aMessageString = "kVisualPluginDeactivateMessage";
			break;
  		case kVisualPluginPulseMessage:
			aMessageString = "kVisualPluginPulseMessage";
			break;
  		case kVisualPluginDrawMessage:
			aMessageString = "kVisualPluginDrawMessage";
			break;
  		case kVisualPluginFrameChangedMessage:
			aMessageString = "kVisualPluginFrameChangedMessage";
			break;
  		case kVisualPluginPlayMessage:
			aMessageString = "kVisualPluginPlayMessage";
			break;
  		case kVisualPluginChangeTrackMessage:
			aMessageString = "kVisualPluginChangeTrackMessage";
			break;
  		case kVisualPluginSetPositionMessage:
			aMessageString = "kVisualPluginSetPositionMessage";
			break;
  		case kVisualPluginStopMessage:
			aMessageString = "kVisualPluginStopMessage";
			break;
  		case kVisualPluginCoverArtMessage:
			aMessageString = "kVisualPluginCoverArtMessage";
			break;
  		case kVisualPluginDisplayChangedMessage:
			aMessageString = "kVisualPluginDisplayChangedMessage";
			break;
        default:
			aMessageString = "unknownVisualPluginMessage";
	}
	strcpy(anOutString, aMessageString);
}


bool VisualMain::storeAudioTrackMetadata(const ITTrackInfo* const someTrackMetadata, bool isAudioStream) 
{
	
	const uint16* trackName = someTrackMetadata->name;
	if (trackName) 
    {
		trackName += 1;
		VisualMainAction::setAudioTrackName((void*)trackName, (int)someTrackMetadata->name[0]);
	}
	
	const uint16* trackArtist = someTrackMetadata->artist;
	if (trackArtist) 
    {
		trackArtist += 1;
		VisualMainAction::setAudioTrackArtistName((void*)trackArtist, (int)someTrackMetadata->artist[0]);
	}
	
	const uint16* trackAlbum = someTrackMetadata->album;
	if (trackAlbum) 
    {
		trackAlbum += 1;
		VisualMainAction::setAudioTrackAlbumName((void*)trackAlbum, (int)someTrackMetadata->album[0]);
	}

	const uint16* trackComposer = someTrackMetadata->composer;
	if (trackComposer) 
    {
		trackComposer += 1;
		VisualMainAction::setAudioTrackComposer((void*)trackComposer, (int)someTrackMetadata->composer[0]);
	}
	
    const uint16* year = &(someTrackMetadata->year);
	if (year) 
    {
        if (*year > 0)
        {
            VisualMainAction::setAudioTrackYear(*year);
        }
	}

	VisualMainAction::setAudioDataIsStream(isAudioStream);

	bool audioTrackDidChange = VisualMainAction::analyzeCurrentlySetMetadata();

	if (!isAudioStream) 
    {
        const UInt64* sizeInBytes = &(someTrackMetadata->sizeInBytes);
        if (sizeInBytes)
        {
            if (*sizeInBytes > 0)
            {
                VisualMainAction::setAudioTrackSizeInBytes(static_cast<int>(*sizeInBytes));
            }
        }
        const UInt32* totalTimeInMS = &(someTrackMetadata->totalTimeInMS);
        if (totalTimeInMS)
        {
            if (*totalTimeInMS > 0)
            {
                VisualMainAction::setTotalTimeOfCurrentTrack(static_cast<int>(*totalTimeInMS));
            }
        }
        const UInt32* startTimeInMS = &(someTrackMetadata->startTimeInMS);
        if (startTimeInMS)
        {
            if (*startTimeInMS > 0)
            {
                VisualMainAction::setStartTimeOfCurrentTrack(static_cast<int>(*startTimeInMS));
            }
        }
        const UInt32* stopTimeInMS = &(someTrackMetadata->stopTimeInMS);
        if (stopTimeInMS)
        {
            if (*stopTimeInMS > 0)
            {
                VisualMainAction::setStopTimeOfCurrentTrack(static_cast<int>(*stopTimeInMS));
            }
        }
	}
	
	return audioTrackDidChange;
	
}


void VisualMain::storeAudioStreamMetadata(const ITStreamInfo* const someStreamMetadata) 
{
	
	const uint16* streamTitle = someStreamMetadata->streamTitle;
	if (streamTitle) 
    {
		streamTitle += 1;
		VisualMainAction::setAudioStreamTitle(streamTitle, (int)someStreamMetadata->streamTitle[0]);
	}

	const uint16* streamMessage = someStreamMetadata->streamMessage;
	if (streamMessage) 
    {
		streamMessage += 1;
		VisualMainAction::setAudioStreamMessage(streamMessage, (int)someStreamMetadata->streamMessage[0]);
	}

	const uint16* streamURL = someStreamMetadata->streamURL;
	if (streamURL) 
    {
		streamURL += 1;
		VisualMainAction::setAudioStreamURL(streamURL, (int)someStreamMetadata->streamURL[0]);
	}
    
    const uint16* streamName = someStreamMetadata->streamName;
	if (streamName) 
    {
		streamName += 1;
		VisualMainAction::setAudioStreamName(streamName, (int)someStreamMetadata->streamName[0]);
	}

	VisualMainAction::setTotalTimeOfCurrentTrack(0);
	
}


OSStatus VisualMain::VisualPluginHandler(OSType aMessage, VisualPluginMessageInfo* aVisualPluginMessageInfo, void* aRefCon)
{

	bool debug = false;

    OSStatus status;
    
    status = noErr;

	if (debug == true) 
    {
		static bool firstTimeRun = true;
		static OSType oneBeforePrevMessage;
		static OSType prevMessage;
		static OSType messageWriteLogHistory[3];
		size_t curr = 2;
		size_t prev = 1;
		size_t oneBeforePrev = 0;
		if (firstTimeRun == true) 
        {
			oneBeforePrevMessage = '    ';
			prevMessage = '    ';
			messageWriteLogHistory[oneBeforePrev] = '    ';
			messageWriteLogHistory[prev] = '    ';
			messageWriteLogHistory[curr] = '    ';
			firstTimeRun = false;
		}
		bool doWriteLog = true;
		if (aMessage != messageWriteLogHistory[curr]) 
        {
			// do not log alternating messages (like render/idle/update messages during render loop)
			if (aMessage == messageWriteLogHistory[prev] && prevMessage == messageWriteLogHistory[curr]) doWriteLog = false;
			if (aMessage == messageWriteLogHistory[oneBeforePrev] && prevMessage == messageWriteLogHistory[curr] && oneBeforePrevMessage == messageWriteLogHistory[prev]) doWriteLog = false;
			if (aMessage == messageWriteLogHistory[prev] && prevMessage == messageWriteLogHistory[oneBeforePrev] && oneBeforePrevMessage == messageWriteLogHistory[curr]) doWriteLog = false;
			if (doWriteLog == true) 
            {
				char visualPluginMessage[64];
				char logStr[128];
				VisualMain::convertVisualPluginMessageToString(aMessage, visualPluginMessage);
				sprintf(logStr, "VisualPluginHandler: message: %s", visualPluginMessage);
				VisualMainAction::writeLogEntry(logStr);
				messageWriteLogHistory[oneBeforePrev] = messageWriteLogHistory[prev];
				messageWriteLogHistory[prev] = messageWriteLogHistory[curr];
				messageWriteLogHistory[curr] = aMessage;
				oneBeforePrevMessage = prevMessage;
				prevMessage = aMessage;
			}
		}
	}

	VisualMainAction::updateTiming();

    switch (aMessage) 
    {
	
        case kVisualPluginInitMessage:
        {
			// This message is sent right after you register your plugin with PlayerRegisterVisualPlugin.
			// The plugin should do minimal memory allocations here.

			VisualSignature::setAppCookie(aVisualPluginMessageInfo->u.initMessage.appCookie);
			VisualSignature::setAppProc(aVisualPluginMessageInfo->u.initMessage.appProc);

#if TARGET_OS_MAC
            // appVersion value on Windows not set
			VisualMainAction::setAppVersion(aVisualPluginMessageInfo->u.initMessage.appVersion.majorRev, aVisualPluginMessageInfo->u.initMessage.appVersion.minorAndBugRev);
#endif
            VisualMainAction::setVisualizerAPIVersion(aVisualPluginMessageInfo->u.initMessage.messageMajorVersion, aVisualPluginMessageInfo->u.initMessage.messageMinorVersion);
			
			VisualMainAction::handleInitMessage(kVisualMaxDataChannels, kVisualNumWaveformEntries, kVisualNumSpectrumEntries);
			
			aVisualPluginMessageInfo->u.initMessage.refCon = NULL;
        }
            break;

        case kVisualPluginCleanupMessage:
			// This message is sent when iTunes is about to quit. 
            // You should free any resources allocated by your visual plugin at this time.

            VisualMainAction::closeThreads();
			VisualMainAction::closePersistentStore();

            break;
                
        case kVisualPluginEnableMessage:
        case kVisualPluginDisableMessage:
			// iTunes currently enables all loaded visual plugins. 
            // Your plugin should simply return noErr for these messages.
			
			break;

        case kVisualPluginIdleMessage:
			// This message is sent periodically if the plugin requests idle messages. 
            // Do this by setting the kVisualWantsIdleMessages option in the PlayerRegisterVisualPluginMessage.options field. 
            // Your plugin should NOT draw at this time, it is for internal book-keeping only.
			
            break;

        case kVisualPluginConfigureMessage:
			// This message is sent when the user picks the Options... item from the Visualizer sub-menu of the View menu. 
            // Enable the Options... menu item (and this message) by setting the kVisualWantsConfigure option 
            // in the PlayerRegisterVisualPluginMessage.options field.

			VisualMainAction::showConfigurationDialog();

			break;

        case kVisualPluginActivateMessage:
        {
			// Sent when visual effects are turned on.
			// At this point, the plugin should allocate any large buffers it needs.
            VisualMainAction::setPlatformView(aVisualPluginMessageInfo->u.activateMessage.view);
				
            bool isFullscreen = ((aVisualPluginMessageInfo->u.activateMessage.options & kWindowIsFullScreen) > 0);
            VisualMainAction::handleShowWindowMessage(isFullscreen);
#if TARGET_OS_MAC
            VisualMainAction::startShow();
#endif
        }

            break;

        case kVisualPluginDeactivateMessage:
			// This message is sent when visual effects are turned off. 
            // Your plugin should free any large buffers allocated in the course of rendering here. 
            // It should also forget about the VISUAL_PLATFORM_VIEW previously sent in the activate message 
            // as it will no longer exist after you return from handling this message.

			VisualMainAction::handleHideWindowMessage();
			
			VisualMainAction::closeThreads();
            
            break;
            
        case kVisualPluginFrameChangedMessage:
            // This message is sent when the user resizes the iTunes window. 
            // Your plugin should adjust its drawing rectangle accordingly 
            // based on the current frame value of the VISUAL_PLATFORM_VIEW previously sent in the activate message.
            
            VisualMainAction::handleCanvasReshape();
            
            break;
            
        case kVisualPluginWindowChangedMessage:
            // This message is sent when the visualizer is moved to a new window (e.g. from a normal window to fullscreen).
        {
            bool isFullscreen = ((aVisualPluginMessageInfo->u.windowChangedMessage.options & kWindowIsFullScreen) > 0);
            VisualMainAction::handleSetWindowMessage(isFullscreen);
        }
            
            break;
            
        case kVisualPluginPulseMessage:
			// This message is sent periodically when the visualizer is active, at a rate specified during registration. 
            // The intent of this message is to supply data fast enough for your plugin to perform calculations on the data 
            // and update its internal drawing state. While it is safe to draw at this time, it is not recommended. 
            // It is preferred that you wait until the draw message is sent before doing any drawing.

            if (aVisualPluginMessageInfo->u.pulseMessage.renderData)
            {
                VisualMainAction::handlePulseMessage(aVisualPluginMessageInfo->u.pulseMessage.currentPositionInMS, aVisualPluginMessageInfo->u.pulseMessage.timeStampID, aVisualPluginMessageInfo->u.pulseMessage.renderData->numWaveformChannels, kVisualNumWaveformEntries, (char*)(aVisualPluginMessageInfo->u.pulseMessage.renderData->waveformData), aVisualPluginMessageInfo->u.pulseMessage.renderData->numSpectrumChannels, kVisualNumSpectrumEntries, (char*)(aVisualPluginMessageInfo->u.pulseMessage.renderData->spectrumData));
			}
#if TARGET_OS_WIN
            // on Mac we use a rendering thread via CVDisplayLink
			VisualMainAction::renderAction();
#endif
            
            aVisualPluginMessageInfo->u.pulseMessage.newPulseRateInHz = 30;
			
            break;

		case kVisualPluginDrawMessage:
            // This message is sent when it is time for the plugin to draw. 
            // This message is synchronized with the OS and is sent when the OS decides 
            // it is time to redraw the region of the window covered by the visuals view.

#if TARGET_OS_WIN
            // on Mac we use a rendering thread via CVDisplayLink
			VisualMainAction::renderAction();
#endif

			break;

		case kVisualPluginDisplayChangedMessage:
			// This message is sent when something about the display environment (bit depth, for example) has changed. 
            // If your plugin handles the event it should return noErr. Otherwise it should return unimpErr
			
            status = unimpErr;
            
            break;

        case kVisualPluginPlayMessage:
        {
			// This message is sent when iTunes starts playing a track. 
            // Your plugin should copy any track info it wants to display. 
            // Also, when streaming an internet radio station the track info contains the name (and meta data) for the radio station, 
            // and the song name (and its meta data) is in the ITStreamInfo field. 
            // Your plugin should not draw at this time, a draw message will be sent as soon as possible.
			
            VisualMainAction::resetVisualAudioLabDataIfNotPaused();
            VisualMainAction::resetCurrAudioMetaData();
        
            bool isAudioStream = false;
            if (aVisualPluginMessageInfo->u.playMessage.streamInfo) 
            {
                isAudioStream = (bool)(aVisualPluginMessageInfo->u.playMessage.streamInfo->streamTitle[0] > 0 ||
                                        aVisualPluginMessageInfo->u.playMessage.streamInfo->streamMessage[0] > 0 ||
                                        aVisualPluginMessageInfo->u.playMessage.streamInfo->streamURL[0] > 0);
            }

            if (isAudioStream) 
            {
                VisualMain::storeAudioStreamMetadata(const_cast<const ITStreamInfo*>(aVisualPluginMessageInfo->u.playMessage.streamInfo));
            }
            bool trackDidChange = VisualMain::storeAudioTrackMetadata(aVisualPluginMessageInfo->u.playMessage.trackInfo, isAudioStream);
                        
            VisualMainAction::handleAudioPlayStartedEvent(trackDidChange);
			    
        }

            break;

        case kVisualPluginChangeTrackMessage:
        {
			// This message is sent when the information about a track changes, e.g., when the user edits track info, 
            // or when iTunes begins playing a different track. Your plugin should copy any track info it wants to display 
            // but should not draw at this time, a draw message will be sent as soon as possible. 
            // Also, when streaming an internet radio station the track info contains the name (and meta data) for the radio station, 
            // and the song name (and its meta data) is in the ITStreamInfo field.

            VisualMainAction::resetCurrAudioMetaData();
            
            bool isAudioStream = false;
            if (aVisualPluginMessageInfo->u.changeTrackMessage.streamInfo) 
            {
                isAudioStream = (bool)(aVisualPluginMessageInfo->u.changeTrackMessage.streamInfo->streamTitle[0] > 0 ||
                                        aVisualPluginMessageInfo->u.changeTrackMessage.streamInfo->streamMessage[0] > 0 ||
                                        aVisualPluginMessageInfo->u.changeTrackMessage.streamInfo->streamURL[0] > 0);
            }

            if (isAudioStream) 
            {
                if (aVisualPluginMessageInfo->u.changeTrackMessage.streamInfo) 
                {
                    VisualMain::storeAudioStreamMetadata(const_cast<const ITStreamInfo*>(aVisualPluginMessageInfo->u.changeTrackMessage.streamInfo));
                } 
            }
            bool trackDidChange = VisualMain::storeAudioTrackMetadata(aVisualPluginMessageInfo->u.changeTrackMessage.trackInfo, isAudioStream);
            
            if (trackDidChange) 
            {
                VisualMainAction::resetVisualAudioLabDataIfNotPaused();
                VisualMainAction::handleAudioPlayStartedEvent(trackDidChange);
            }
        }

            break;

        case kVisualPluginStopMessage:
			// This message is sent when the music stops playing.

            VisualMainAction::handleAudioPlayStoppedEvent();
			
            break;
        
        case kVisualPluginSetPositionMessage:
			// This message is sent when iTunes changes the elapsed time position within a track. 
            // A plugin that shows the elapsed time would use this.

			VisualMainAction::setTrackPlayPositionInMS(aVisualPluginMessageInfo->u.setPositionMessage.positionTimeInMS);
			
            break;
            
        case kVisualPluginCoverArtMessage:
            // This message is sent in response to a request for the currently playing tracks cover art.
            // A platform-specific pointer to the cover art data. 
            // On the Mac, this is a CFDataRef. On Windows, it is a LPCVOID pointer. 
            // The data is disposed after this call returns so the plugin must not save off the pointer and expect it to be valid later. 
            // If the plugin wants to keep the data around for later, it needs to retain it (Mac) or copy it (Windows). 
            // If the currently playing track has no artwork, this pointer will be nil.
            
            if (aVisualPluginMessageInfo->u.coverArtMessage.coverArt)
            {
                VisualMainAction::postAlbumCoverArtworkOfCurrentAudioData(aVisualPluginMessageInfo->u.coverArtMessage.coverArt, aVisualPluginMessageInfo->u.coverArtMessage.coverArtSize, aVisualPluginMessageInfo->u.coverArtMessage.coverArtFormat);
            }
            
            break;

        default:
		
            char errStr[512];
            sprintf(errStr, "Err: Unhandled message %d in file: %s (line: %d) [%s])", (int)aMessage, __FILE__, __LINE__, __FUNCTION__);
            VisualMainAction::writeLogEntry(errStr);
            
            status = unimpErr;
			
            break;
			
    }
	
    return status;	

}


OSStatus VisualMain::RegisterVisualPlugin(PluginMessageInfo* aPluginMessageInfo) 
{

    PlayerMessageInfo aPlayerMessageInfo;
    
    memset(&aPlayerMessageInfo.u.registerVisualPluginMessage, 0, sizeof(aPlayerMessageInfo.u.registerVisualPluginMessage));
	
#if HasConfigurationWindow
	aPlayerMessageInfo.u.registerVisualPluginMessage.options = (kVisualWantsConfigure | kVisualWantsIdleMessages | kVisualUsesOnly3D | kVisualUsesSubview);
#else
    aPlayerMessageInfo.u.registerVisualPluginMessage.options = (kVisualWantsIdleMessages | kVisualUsesOnly3D | kVisualUsesSubview);
#endif

	long numberOfCharactersOfVisualizerShowNameUnicode = 0;
	uint16* visualizerShowNameUnicode = VisualConfiguration::getVisualizerShowName(numberOfCharactersOfVisualizerShowNameUnicode);
	aPlayerMessageInfo.u.registerVisualPluginMessage.name[0] = (UniChar)numberOfCharactersOfVisualizerShowNameUnicode;
	for (long i = 0; i < numberOfCharactersOfVisualizerShowNameUnicode; i++) 
    {
		aPlayerMessageInfo.u.registerVisualPluginMessage.name[i + 1] = visualizerShowNameUnicode[i];
	}

	const UInt8 stage = 0x80; // (dev = 0x20, alpha = 0x40, beta = 0x60, final = 0x80)
	SetNumVersion(&aPlayerMessageInfo.u.registerVisualPluginMessage.pluginVersion, VisualConfiguration::getMajorReleaseNumberAsBCD(), VisualConfiguration::getMinorReleaseNumberAsBCD(), stage, VisualConfiguration::getReleaseRevisionNumber());
    
    aPlayerMessageInfo.u.registerVisualPluginMessage.handler = (VisualPluginProcPtr)VisualMain::VisualPluginHandler;

    aPlayerMessageInfo.u.registerVisualPluginMessage.registerRefCon = 0;
	
	OSType visualPluginCreator = 'hook';
	aPlayerMessageInfo.u.registerVisualPluginMessage.creator = visualPluginCreator;
    
    aPlayerMessageInfo.u.registerVisualPluginMessage.pulseRateInHz = 5;	// update my state 5 times a second as long as iTunes is not playing
    aPlayerMessageInfo.u.registerVisualPluginMessage.numWaveformChannels = 2;
    aPlayerMessageInfo.u.registerVisualPluginMessage.numSpectrumChannels = 2;
    
    aPlayerMessageInfo.u.registerVisualPluginMessage.minWidth = 64;
    aPlayerMessageInfo.u.registerVisualPluginMessage.minHeight = 64;
    aPlayerMessageInfo.u.registerVisualPluginMessage.maxWidth = 0; // no max width limit
    aPlayerMessageInfo.u.registerVisualPluginMessage.maxHeight = 0; // no max height limit
    
    OSStatus status = PlayerRegisterVisualPlugin(aPluginMessageInfo->u.initMessage.appCookie, aPluginMessageInfo->u.initMessage.appProc, &aPlayerMessageInfo);
    
    return status;
	
}


#if TARGET_OS_MAC
OSStatus iTunesPluginMainMachO(OSType aMessage, PluginMessageInfo* aPluginMessageInfo, void* aRefCon)
#endif
#if TARGET_OS_WIN
OSStatus iTunesPluginMain(OSType aMessage, PluginMessageInfo* aPluginMessageInfo, void* aRefCon)
#endif
{
    OSStatus status;
    
    switch (aMessage) 
    {
        case kPluginInitMessage:
            status = VisualMain::RegisterVisualPlugin(aPluginMessageInfo);
            break;
            
        case kPluginIdleMessage:
            status = noErr;
            break;
                
        case kPluginCleanupMessage:
            status = noErr;
            break;

        case kPluginPrepareToQuitMessage:
            status = noErr;
            break;
                
        default:
            status = unimpErr;
            break;
    }
    
    return status;
}

