/*
 * Project: VizKit
 * Version: 3.3.5
 
 * Date: 20141018
 * File: OptionsDialogWindowController.m
 *
 */

/***************************************************************************
 
 Copyright (c) 2004-2014 Heiko Wichmann (http://www.imagomat.de/vizkit)
 
 
 This software is provided 'as-is', without any expressed or implied warranty. 
 In no event will the authors be held liable for any damages
 arising from the use of this software.
 
 Permission is granted to anyone to use this software for any purpose,
 including commercial applications, and to alter it and redistribute it
 freely, subject to the following restrictions:
 
 1. The origin of this software must not be misrepresented; 
 you must not claim that you wrote the original software. 
 If you use this software in a product, an acknowledgment 
 in the product documentation would be appreciated 
 but is not required.
 
 2. Altered source versions must be plainly marked as such, 
 and must not be misrepresented as being the original software.
 
 3. This notice may not be removed or altered from any source distribution.
 
 ***************************************************************************/

#import "OptionsDialogWindowController.h"

#include "VisualCustomization.h"
#include "VisualErrorHandling.h"
#include "VisualPreferences.h"
#include "VisualPlayerState.h"
#include "VisualFontManager.H"
#include "VisualMainAction.h"
#include "VisualConfigurationDialog.h"


// file type of OptionsDialogWindowController.m is set to "Objective-C++ source"


@interface OptionsDialogWindowController()

@property(nonatomic, retain) NSFont *currFont;

@property(nonatomic, assign) NSTabView *tabView;
@property(nonatomic, assign) NSTextField *aboutTitleField;
@property(nonatomic, assign) NSTextField *aboutTextField;
@property(nonatomic, assign) NSTextField *fontNameTextField;
@property(nonatomic, assign) NSButton *checkForUpdateOnStartupCheckBoxButton;

@end


@implementation OptionsDialogWindowController


@synthesize tabView;
@synthesize aboutTitleField;
@synthesize aboutTextField;
@synthesize fontNameTextField;
@synthesize checkForUpdateOnStartupCheckBoxButton;
@synthesize currFont;

- (id)init 
{
    self = [super initWithWindowNibName:@"VizKitOptionsDialog"];
    if (self) 
    {
        //
    }
    else
    {
        NSLog(@"%@ - %@: err: failed to init", NSStringFromClass([self class]), NSStringFromSelector(_cmd));
    }
    return self;
}


- (void)dealloc 
{
    [currFont release];
    
    [super dealloc];
}


- (void)windowDidLoad
{
    [super windowDidLoad];
    
    // about
    [aboutTitleField setStringValue:@"VizKit"];
    [aboutTextField setStringValue:@"Version 3.3.5\nhttp://www.imagomat.de/vizkit\n@ 2004-2014 Heiko Wichmann"];
    
    // fonts
    VizKit::VisualFontManager *theVisualFontManager = VizKit::VisualFontManager::getInstance();
    VizKit::VisualFontActivationResult aFontActivationResult = theVisualFontManager->activateFont("Devroye");
    if (aFontActivationResult == VizKit::fontActivationUnableToActivateFont)
    {
        NSLog(@"err: %@ - %@: Unable to activate font %s", NSStringFromClass([self class]), NSStringFromSelector(_cmd), "Devroye");
    }
    char aCurrFontNameUtf8String[512];
    VizKit::VisualPreferences::getValue(VizKit::VisualPreferences::kTrackInfoFont, aCurrFontNameUtf8String);
    NSString *aCurrFontName = [NSString stringWithUTF8String:aCurrFontNameUtf8String];
    NSFont *aCurrFont = [NSFont fontWithName:aCurrFontName size:12.0];
    if (aCurrFont)
    {
        [self.fontNameTextField setStringValue:[aCurrFont displayName]];
    }
    else
    {
        [self.fontNameTextField setStringValue:@"---"];
    }
    
    // update
    bool doCheckForUpdate = VizKit::VisualPreferences::getValue(VizKit::VisualPreferences::kCheckForUpdate);
    if (doCheckForUpdate == true) 
    {
        [checkForUpdateOnStartupCheckBoxButton setState:NSOnState];
    } 
    else 
    {
        [checkForUpdateOnStartupCheckBoxButton setState:NSOffState];
    }
    
    int aLastTabIndex = VizKit::VisualPreferences::getValue(VizKit::VisualPreferences::kPreferencePane);
    if (aLastTabIndex < (int)([tabView tabViewItems].count - 1))
    {
        [tabView selectTabViewItem:[tabView tabViewItemAtIndex:aLastTabIndex]];
    }
    else
    {
        [tabView selectTabViewItem:[tabView tabViewItemAtIndex:0]];
    }
    [tabView setHidden:NO];
}


- (void)windowWillClose:(NSNotification *)notification
{    
    [[NSFontPanel sharedFontPanel] close];
    NSFontManager *theFontManager = [NSFontManager sharedFontManager];
    [theFontManager setTarget:nil];
 
    VizKit::VisualConfigurationDialog::resetOptionsDialogWindowController();
}


- (void)tabView:(NSTabView *)aTabView didSelectTabViewItem:(NSTabViewItem *)aTabViewItem
{
    NSInteger aSelectedTabIndex = [aTabView indexOfTabViewItem:aTabViewItem];
    VizKit::VisualPreferences::setValue(VizKit::VisualPreferences::kPreferencePane, (int)aSelectedTabIndex);
	VizKit::VisualPreferences::storeValues();
}


- (IBAction)toggleCheckForUpdateOnStartup:(id)aSender
{
    NSInteger aCurrState = [aSender state];
    if (aCurrState == NSOffState)
    {
        VizKit::VisualPreferences::setValue(VizKit::VisualPreferences::kCheckForUpdate, false);
    }
    else
    {
        VizKit::VisualPreferences::setValue(VizKit::VisualPreferences::kCheckForUpdate, true);
    }
    VizKit::VisualPreferences::storeValues();
}


- (IBAction)selectTrackInfoFont:(id)aSender
{
    char aCurrFontNameUtf8String[512];
    VizKit::VisualPreferences::getValue(VizKit::VisualPreferences::kTrackInfoFont, aCurrFontNameUtf8String);
    NSString *aCurrFontName = [NSString stringWithUTF8String:aCurrFontNameUtf8String];
    self.currFont = [NSFont fontWithName:aCurrFontName size:12.0];
    
    NSFontManager *theFontManager = [NSFontManager sharedFontManager];
    [theFontManager setTarget:self];
    [theFontManager setSelectedFont:self.currFont isMultiple:NO];
    [theFontManager orderFrontFontPanel:self];
}


- (NSUInteger)validModesForFontPanel:(NSFontPanel *)aFontPanel
{
    return (NSFontPanelFaceModeMask | NSFontPanelCollectionModeMask);
}


- (void)changeFont:(id)aSender
{
    NSFont* aNewFont = [aSender convertFont:self.currFont];
    if (aNewFont)
    {
        NSString *aNewFontName = [aNewFont fontName];
        const char *aFontNameUtf8String = [aNewFontName UTF8String];
        VizKit::VisualPreferences::setValue(VizKit::VisualPreferences::kTrackInfoFont, aFontNameUtf8String);
        VizKit::VisualPreferences::storeValues();
        self.currFont = aNewFont;
        [self.fontNameTextField setStringValue:[aNewFont displayName]];
        
        VizKit::VisualPlayerState* theVisualPlayerState = VizKit::VisualPlayerState::getInstance();
        if (theVisualPlayerState->getPlayerShowMode() & VizKit::kIsShowing) {
            VizKit::VisualMainAction::postTrackInfoOfCurrentAudioData();
        }
    }
    else
    {
        NSLog(@"ERR: No font (%s)", __PRETTY_FUNCTION__);
    }
}

@end
